@@ -99,6 +99,19 @@ pan_stream_close(struct vsb *vsb, stream_close_t sc)
 
 /*--------------------------------------------------------------------*/
 
+static void
+pan_storage(struct vsb *vsb, const char *n, const struct stevedore *stv)
+{
+
+	if (stv != NULL && stv->magic == STEVEDORE_MAGIC)
+		VSB_printf(vsb, "%s = %s(%s,%s),\n",
+		    n, stv->name, stv->ident, stv->vclname);
+	else
+		VSB_printf(vsb, "%s = %p,\n", n, stv);
+}
+
+/*--------------------------------------------------------------------*/
+
 #define N_ALREADY 256
 static const void *already_list[N_ALREADY];
 static int already_idx;
@@ -400,6 +413,7 @@ pan_busyobj(struct vsb *vsb, const struct busyobj *bo)
 
 	// timeouts/timers/acct/storage left out
 
+	pan_storage(vsb, "storage", bo->storage);
 	VDI_Panic(bo->director_req, vsb, "director_req");
 	if (bo->director_resp == bo->director_req)
 		VSB_cat(vsb, "director_resp = director_req,\n");
@@ -438,7 +452,7 @@ pan_req(struct vsb *vsb, const struct req *req)
 	if (PAN_dump_struct(vsb, req, REQ_MAGIC, "req"))
 		return;
 	xp = req->transport;
-	VSB_printf(vsb, "vxid = %u, transport = %s", VXID(req->vsl->wid),
+	VSB_printf(vsb, "vxid = %ju, transport = %s", VXID(req->vsl->wid),
 	    xp == NULL ? "NULL" : xp->name);
 
 	if (xp != NULL && xp->req_panic != NULL) {
@@ -465,6 +479,16 @@ pan_req(struct vsb *vsb, const struct req *req)
 	VSB_printf(vsb, "restarts = %u, esi_level = %u,\n",
 	    req->restarts, req->esi_level);
 
+	VSB_printf(vsb, "vary_b = %p, vary_e = %p,\n",
+	    req->vary_b, req->vary_e);
+
+	VSB_printf(vsb, "d_ttl = %f, d_grace = %f,\n",
+	    req->d_ttl, req->d_grace);
+
+	pan_storage(vsb, "storage", req->storage);
+
+	VDI_Panic(req->director_hint, vsb, "director_hint");
+
 	if (req->sp != NULL)
 		pan_sess(vsb, req->sp);
 
@@ -529,7 +553,7 @@ pan_sess(struct vsb *vsb, const struct sess *sp)
 
 	if (PAN_dump_struct(vsb, sp, SESS_MAGIC, "sess"))
 		return;
-	VSB_printf(vsb, "fd = %d, vxid = %u,\n",
+	VSB_printf(vsb, "fd = %d, vxid = %ju,\n",
 	    sp->fd, VXID(sp->vxid));
 	VSB_printf(vsb, "t_open = %f,\n", sp->t_open);
 	VSB_printf(vsb, "t_idle = %f,\n", sp->t_idle);
