@@ -49,7 +49,6 @@
 #include "hash/hash_slinger.h"
 #include "http1/cache_http1.h"
 #include "storage/storage.h"
-#include "common/heritage.h"
 #include "vcl.h"
 #include "vct.h"
 #include "vsha256.h"
@@ -153,10 +152,10 @@ Resp_Setup_Deliver(struct req *req)
 	http_ForceField(h, HTTP_HDR_PROTO, "HTTP/1.1");
 
 	if (req->is_hit)
-		http_PrintfHeader(h, "X-Varnish: %u %u", VXID(req->vsl->wid),
+		http_PrintfHeader(h, "X-Varnish: %ju %u", VXID(req->vsl->wid),
 		    ObjGetXID(req->wrk, oc));
 	else
-		http_PrintfHeader(h, "X-Varnish: %u", VXID(req->vsl->wid));
+		http_PrintfHeader(h, "X-Varnish: %ju", VXID(req->vsl->wid));
 
 	/*
 	 * We base Age calculation upon the last timestamp taken during client
@@ -193,7 +192,7 @@ Resp_Setup_Synth(struct req *req)
 
 	http_TimeHeader(h, "Date: ", W_TIM_real(req->wrk));
 	http_SetHeader(h, "Server: Varnish");
-	http_PrintfHeader(h, "X-Varnish: %u", VXID(req->vsl->wid));
+	http_PrintfHeader(h, "X-Varnish: %ju", VXID(req->vsl->wid));
 
 	/*
 	 * For late 100-continue, we suggest to VCL to close the connection to
@@ -767,8 +766,8 @@ cnt_pipe(struct worker *wrk, struct req *req)
 	wrk->stats->s_pipe++;
 	bo = VBO_GetBusyObj(wrk, req);
 	CHECK_OBJ_NOTNULL(bo, BUSYOBJ_MAGIC);
-	VSLb(bo->vsl, SLT_Begin, "bereq %u pipe", VXID(req->vsl->wid));
-	VSLb(req->vsl, SLT_Link, "bereq %u pipe", VXID(bo->vsl->wid));
+	VSLb(bo->vsl, SLT_Begin, "bereq %ju pipe", VXID(req->vsl->wid));
+	VSLb(req->vsl, SLT_Link, "bereq %ju pipe", VXID(bo->vsl->wid));
 	VSLb_ts_busyobj(bo, "Start", W_TIM_real(wrk));
 	THR_SetBusyobj(bo);
 	bo->sp = req->sp;
@@ -776,7 +775,7 @@ cnt_pipe(struct worker *wrk, struct req *req)
 
 	HTTP_Setup(bo->bereq, req->ws, bo->vsl, SLT_BereqMethod);
 	http_FilterReq(bo->bereq, req->http, 0);	// XXX: 0 ?
-	http_PrintfHeader(bo->bereq, "X-Varnish: %u", VXID(req->vsl->wid));
+	http_PrintfHeader(bo->bereq, "X-Varnish: %ju", VXID(req->vsl->wid));
 	http_ForceHeader(bo->bereq, H_Connection, "close");
 
 	if (req->want100cont) {
@@ -861,7 +860,7 @@ cnt_restart(struct worker *wrk, struct req *req)
  *
  * TODO
  * - make restarts == 0 bit re-usable for rollback
- * - remove duplicatation with Req_Cleanup()
+ * - remove duplication with Req_Cleanup()
  */
 
 static void v_matchproto_(req_state_f)
@@ -909,7 +908,7 @@ cnt_recv(struct worker *wrk, struct req *req)
 {
 	unsigned recv_handling;
 	struct VSHA256Context sha256ctx;
-	const char *ci, *cp, *endpname;
+	const char *ci;
 
 	CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
 	CHECK_OBJ_NOTNULL(req, REQ_MAGIC);
@@ -922,13 +921,7 @@ cnt_recv(struct worker *wrk, struct req *req)
 	AZ(isnan(req->t_prev));
 	AZ(isnan(req->t_req));
 
-	ci = SES_Get_String_Attr(req->sp, SA_CLIENT_IP);
-	cp = SES_Get_String_Attr(req->sp, SA_CLIENT_PORT);
-	CHECK_OBJ_NOTNULL(req->sp->listen_sock, LISTEN_SOCK_MAGIC);
-	endpname = req->sp->listen_sock->name;
-	AN(endpname);
-	VSLb(req->vsl, SLT_ReqStart, "%s %s %s", ci, cp, endpname);
-
+	ci = Req_LogStart(wrk, req);
 	http_VSL_log(req->http);
 
 	if (http_CountHdr(req->http0, H_Host) > 1) {
@@ -1112,8 +1105,8 @@ cnt_diag(struct req *req, const char *state)
 
 	CHECK_OBJ_NOTNULL(req, REQ_MAGIC);
 
-	VSLb(req->vsl,  SLT_Debug, "vxid %u STP_%s sp %p vcl %p",
-	    req->vsl->wid, state, req->sp, req->vcl);
+	VSLb(req->vsl,  SLT_Debug, "vxid %ju STP_%s sp %p vcl %p",
+	    VXID(req->vsl->wid), state, req->sp, req->vcl);
 	VSL_Flush(req->vsl, 0);
 }
 
@@ -1160,7 +1153,7 @@ CNT_Request(struct req *req)
 	    req->req_step == R_STP_LOOKUP ||
 	    req->req_step == R_STP_TRANSPORT);
 
-	AN(req->vsl->wid & VSL_CLIENTMARKER);
+	AN(VXID_TAG(req->vsl->wid) & VSL_CLIENTMARKER);
 	AN(req->vcl);
 
 	for (nxt = REQ_FSM_MORE; nxt == REQ_FSM_MORE; ) {
@@ -1192,7 +1185,7 @@ CNT_Request(struct req *req)
 				VCL_Recache(wrk, &req->top->vcl0);
 		}
 		VCL_TaskLeave(ctx, req->privs);
-		AN(req->vsl->wid);
+		assert(!IS_NO_VXID(req->vsl->wid));
 		VRB_Free(req);
 		VRT_Assign_Backend(&req->director_hint, NULL);
 		req->wrk = NULL;
