@@ -125,7 +125,7 @@ VPI_acl_log(VRT_CTX, const char *msg)
 	if (ctx->vsl != NULL)
 		VSLbs(ctx->vsl, SLT_VCL_acl, TOSTRAND(msg));
 	else
-		VSL(SLT_VCL_acl, 0, "%s", msg);
+		VSL(SLT_VCL_acl, NO_VXID, "%s", msg);
 }
 
 int
@@ -722,7 +722,6 @@ VCL_STRING v_matchproto_()
 VRT_IP_string(VRT_CTX, VCL_IP ip)
 {
 	char buf[VTCP_ADDRBUFSIZE];
-	struct vsb vsb[1];
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
 	if (ip == NULL) {
@@ -730,9 +729,7 @@ VRT_IP_string(VRT_CTX, VCL_IP ip)
 		return (NULL);
 	}
 	VTCP_name(ip, buf, sizeof buf, NULL, 0);
-	WS_VSB_new(vsb, ctx->ws);
-	VSB_cat(vsb, buf);
-	return (WS_VSB_finish(vsb, ctx->ws, NULL));
+	return (WS_Copy(ctx->ws, buf, -1));
 }
 
 int
@@ -748,7 +745,7 @@ VRT_INT_string(VRT_CTX, VCL_INT num)
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
 	if (!VRT_INT_is_valid(num))
-		VRT_fail(ctx, "INT overflow converting to string (%jX)",
+		VRT_fail(ctx, "INT overflow converting to string (0x%jx)",
 		    (intmax_t)num);
 	return (WS_Printf(ctx->ws, "%jd", (intmax_t)num));
 }
