@@ -303,6 +303,36 @@ VRT_r_beresp_uncacheable(VRT_CTX)
 
 /*--------------------------------------------------------------------*/
 
+VCL_BYTES
+VRT_r_beresp_transit_buffer(VRT_CTX)
+{
+    struct objcore *oc;
+
+    CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+    CHECK_OBJ_NOTNULL(ctx->bo, BUSYOBJ_MAGIC);
+
+    oc = ctx->bo->fetch_objcore;
+    CHECK_OBJ_NOTNULL(oc, OBJCORE_MAGIC);
+
+    return oc->boc->transit_buffer;
+}
+
+VCL_VOID
+VRT_l_beresp_transit_buffer(VRT_CTX, VCL_BYTES value)
+{
+    struct objcore *oc;
+
+    CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+    CHECK_OBJ_NOTNULL(ctx->bo, BUSYOBJ_MAGIC);
+
+    oc = ctx->bo->fetch_objcore;
+    CHECK_OBJ_NOTNULL(oc, OBJCORE_MAGIC);
+
+    oc->boc->transit_buffer = value;
+}
+
+/*--------------------------------------------------------------------*/
+
 VCL_STRING
 VRT_r_client_identity(VRT_CTX)
 {
@@ -767,7 +797,7 @@ VRT_DO_AGE_R(beresp, ctx->bo->fetch_objcore)
  * [[be]req|sess].xid
  */
 
-VCL_STRING
+VCL_INT
 VRT_r_req_xid(VRT_CTX)
 {
 
@@ -775,12 +805,10 @@ VRT_r_req_xid(VRT_CTX)
 	CHECK_OBJ_NOTNULL(ctx->req, REQ_MAGIC);
 	CHECK_OBJ_NOTNULL(ctx->req->http, HTTP_MAGIC);
 	AN(ctx->req->vsl);
-
-	return (WS_Printf(ctx->req->http->ws, "%u",
-	    VXID(ctx->req->vsl->wid)));
+	return (VXID(ctx->req->vsl->wid));
 }
 
-VCL_STRING
+VCL_INT
 VRT_r_bereq_xid(VRT_CTX)
 {
 
@@ -788,10 +816,10 @@ VRT_r_bereq_xid(VRT_CTX)
 	CHECK_OBJ_NOTNULL(ctx->bo, BUSYOBJ_MAGIC);
 	AN(ctx->bo->vsl);
 
-	return (WS_Printf(ctx->ws, "%u", VXID(ctx->bo->vsl->wid)));
+	return (VXID(ctx->bo->vsl->wid));
 }
 
-VCL_STRING
+VCL_INT
 VRT_r_sess_xid(VRT_CTX)
 {
 	struct sess *sp;
@@ -807,7 +835,7 @@ VRT_r_sess_xid(VRT_CTX)
 	}
 
 	CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);
-	return (WS_Printf(ctx->ws, "%u", VXID(sp->vxid)));
+	return (VXID(sp->vxid));
 }
 
 /*--------------------------------------------------------------------
