@@ -252,25 +252,28 @@ sml_slim(struct worker *wrk, struct objcore *oc)
 static void v_matchproto_(objfree_f)
 sml_objfree(struct worker *wrk, struct objcore *oc)
 {
+	struct storage *st;
 	struct object *o;
 
 	CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
 	CHECK_OBJ_NOTNULL(oc, OBJCORE_MAGIC);
 	sml_slim(wrk, oc);
 	CAST_OBJ_NOTNULL(o, oc->stobj->priv, OBJECT_MAGIC);
-	o->magic = 0;
+	st = o->objstore;
+	CHECK_OBJ_NOTNULL(st, STORAGE_MAGIC);
+	FINI_OBJ(o);
 
 	if (oc->boc == NULL && oc->stobj->stevedore->lru != NULL)
 		LRU_Remove(oc);
 
-	sml_stv_free(oc->stobj->stevedore, o->objstore);
+	sml_stv_free(oc->stobj->stevedore, st);
 
 	memset(oc->stobj, 0, sizeof oc->stobj);
 
 	wrk->stats->n_object--;
 }
 
-static int v_matchproto_(objiterate_f)
+static int v_matchproto_(objiterator_f)
 sml_iterator(struct worker *wrk, struct objcore *oc,
     void *priv, objiterate_f *func, int final)
 {
@@ -321,7 +324,7 @@ sml_iterator(struct worker *wrk, struct objcore *oc,
 	l = 0;
 
 	u = 0;
-	if (boc->len_so_far == 0) {
+	if (boc->fetched_so_far == 0) {
 		ret = func(priv, OBJ_ITER_FLUSH, NULL, 0);
 		if (ret)
 			return (ret);
