@@ -38,8 +38,6 @@
 
 #include "cache/cache_varnishd.h"
 #include "cache/cache_filter.h"
-#include "vcc_interface.h"	// struct wrk_vpi
-
 
 #include "vsa.h"
 #include "vtim.h"
@@ -334,6 +332,15 @@ xyzzy_test_priv_vcl(VRT_CTX, struct vmod_priv *priv)
 	assert(!strcmp(priv_vcl->foo, "FOO"));
 }
 
+VCL_VOID v_matchproto_(td_debug_rot104)
+xyzzy_rot104(VRT_CTX)
+{
+
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+	// This should fail
+	AN(VRT_AddFilter(ctx, &xyzzy_vfp_rot13, &xyzzy_vdp_rot13));
+}
+
 VCL_VOID v_matchproto_(td_debug_rot52)
 xyzzy_rot52(VRT_CTX, VCL_HTTP hp)
 {
@@ -379,7 +386,7 @@ obj_cb(struct worker *wrk, void *priv, struct objcore *oc, unsigned event)
 	}
 
 	/* We cannot trust %p to be 0x... format as expected by m00021.vtc */
-	VSL(SLT_Debug, 0, "Object Event: %s 0x%jx", what,
+	VSL(SLT_Debug, NO_VXID, "Object Event: %s 0x%jx", what,
 	    (intmax_t)(uintptr_t)oc);
 }
 
@@ -393,7 +400,7 @@ xyzzy_register_obj_events(VRT_CTX, struct vmod_priv *priv)
 	AZ(priv_vcl->obj_cb);
 	priv_vcl->obj_cb = ObjSubscribeEvents(obj_cb, priv_vcl,
 		OEV_INSERT|OEV_EXPIRE);
-	VSL(SLT_Debug, 0, "Subscribed to Object Events");
+	VSL(SLT_Debug, NO_VXID, "Subscribed to Object Events");
 }
 
 VCL_VOID v_matchproto_(td_debug_fail)
@@ -456,15 +463,6 @@ event_load(VRT_CTX, struct vmod_priv *priv)
 	priv->methods = priv_vcl_methods;
 
 	AZ(VRT_AddFilter(ctx, &xyzzy_vfp_rot13, &xyzzy_vdp_rot13));
-
-	// This should fail
-	AN(VRT_AddFilter(ctx, &xyzzy_vfp_rot13, &xyzzy_vdp_rot13));
-	// Reset the error, we know what we're doing.
-	ctx->vpi->handling = 0;
-
-	VRT_RemoveFilter(ctx, &xyzzy_vfp_rot13, &xyzzy_vdp_rot13);
-	AZ(VRT_AddFilter(ctx, &xyzzy_vfp_rot13, &xyzzy_vdp_rot13));
-
 	AZ(VRT_AddFilter(ctx, NULL, &xyzzy_vdp_pedantic));
 	return (0);
 }
@@ -531,7 +529,7 @@ event_warm(VRT_CTX, const struct vmod_priv *priv)
 	const char *vcl_name = VCL_Name(ctx->vcl);
 
 	// Using VSLs for coverage
-	VSLs(SLT_Debug, 0, TOSTRANDS(2, vcl_name, ": VCL_EVENT_WARM"));
+	VSLs(SLT_Debug, NO_VXID, TOSTRANDS(2, vcl_name, ": VCL_EVENT_WARM"));
 
 	AN(ctx->msg);
 	if (cache_param->max_esi_depth == 42) {
@@ -579,7 +577,7 @@ create_cold_backend(VRT_CTX)
 	INIT_OBJ(be, VRT_BACKEND_MAGIC);
 	be->endpoint = vep;
 	be->vcl_name = "doomed";
-	return (VRT_new_backend(ctx, be));
+	return (VRT_new_backend(ctx, be, NULL));
 }
 
 static int
@@ -592,7 +590,7 @@ event_cold(VRT_CTX, const struct vmod_priv *priv)
 
 	CAST_OBJ_NOTNULL(priv_vcl, priv->priv, PRIV_VCL_MAGIC);
 
-	VSL(SLT_Debug, 0, "%s: VCL_EVENT_COLD", VCL_Name(ctx->vcl));
+	VSL(SLT_Debug, NO_VXID, "%s: VCL_EVENT_COLD", VCL_Name(ctx->vcl));
 
 	VRT_DelDirector(&priv_vcl->be);
 
@@ -835,7 +833,7 @@ mylog(struct vsl_log *vsl, enum VSL_tag_e tag,  const char *fmt, ...)
 	if (vsl != NULL)
 		VSLbv(vsl, tag, fmt, ap);
 	else
-		VSLv(tag, 0, fmt, ap);
+		VSLv(tag, NO_VXID, fmt, ap);
 	va_end(ap);
 }
 
@@ -952,12 +950,9 @@ xyzzy_catflap(VRT_CTX, VCL_ENUM type)
 	req = ctx->req;
 	CHECK_OBJ_NOTNULL(req, REQ_MAGIC);
 	XXXAZ(req->vcf);
-	req->vcf = WS_Alloc(req->ws, sizeof *req->vcf);
-	if (req->vcf == NULL) {
-		VRT_fail(ctx, "WS_Alloc failed in debug.catflap()");
+	WS_TASK_ALLOC_OBJ(ctx, req->vcf, VCF_MAGIC);
+	if (req->vcf == NULL)
 		return;
-	}
-	INIT_OBJ(req->vcf, VCF_MAGIC);
 	if (type == VENUM(first) || type == VENUM(miss)) {
 		req->vcf->func = xyzzy_catflap_simple;
 		req->vcf->priv = TRUST_ME(type);
